#include <bits/stdc++.h>

using namespace std;

#define pb push_back
#define eb emplace_back

using ull = unsigned long long;
using vi = vector<int>;
using str = string;

const int K = 26, Q = 300043, LG = 20;

struct DSU {
    vi p;

    DSU(int n) {
        p.resize(n);
        iota(p.begin(), p.end(), 0);
    }

    int get(int u) {
        return u == p[u] ? u : p[u] = get(p[u]);
    }

    void unite(int u, int v) {
        u = get(u);
        v = get(v);
        if (u == v) {
            return;
        }
        p[u] = p[v];
    }
};

struct Aho {
    struct Vertex {
        map<char, int> next;
        int terminal = -1;
        int p = -1;
        char pch;
        int link = -1;
        int dp = -1;
    };

    vector<Vertex> t;

    Aho() : t(1) {}

    void add_string(const string& s, int id) {
        int cur = 0;
        for (auto& ch : s) {
            auto it = t[cur].next.find(ch);
            if (it == t[cur].next.end()) {
                t.eb();
                t.back().p = cur;
                t.back().pch = ch;
                t[cur].next[ch] = t.size() - 1;
                cur = t.size() - 1;
            } else {
                cur = it->second;
            }
        }
        t[cur].terminal = id;
    }

    int get_link(int v) {
        if (t[v].link == -1) {
            if (v == 0 || t[v].p == 0) {
                t[v].link = 0;
            } else {
                int tmp = get_link(t[v].p);
                t[v].link = go(get_link(t[v].p), t[v].pch);
            }
        }
        return t[v].link;
    }

    int go(int v, char ch) {
        auto it = t[v].next.find(ch);
        if (it == t[v].next.end()) {
            int nxt = v == 0 ? 0 : go(get_link(v), ch);
            t[v].next[ch] = nxt;
            return nxt;
        }
        return it->second;
    }

    void calculate() {
        for (int i = 1; i < t.size(); i++) {
            if (t[i].dp != -1) {
                continue;
            }
            if (t[i].terminal != -1) {
                t[i].dp = t[i].terminal;
            } else {
                vi st;
                st.pb(i);
                while (t[st.back()].dp == -1 && t[st.back()].terminal == -1) {
                    st.pb(get_link(st.back()));
                }
                int x = t[st.back()].terminal != -1 ? t[st.back()].terminal : t[st.back()].dp;
                while (!st.empty()) {
                    t[st.back()].dp = x;
                    st.pop_back();
                }
            }
        }
    }

    int get_dp(int x) {
        return t[x].dp;
    }
};

struct Hash {
    ull hash;
    int sz;

    Hash(ull hash = 0, int sz = 0) : hash(hash), sz(sz) { }
};

ull pw[(int)1e6 + 5];

void precalc() {
    pw[0] = 1;
    for (int i = 1; i < (int)1e6 + 5; i++) {
        pw[i] = pw[i - 1] * Q;
    }
}

Hash unite(Hash l, Hash r) {
    Hash res;
    res.hash = l.hash * pw[r.sz] + r.hash;
    res.sz = l.sz + r.sz;
    return res;
}

struct BinUp {
    int to;
    Hash hash;
};

void solve() {
    str t;
    int n, k;
    cin >> t >> n >> k;
    reverse(t.begin(), t.end());
    n += K;
    vector<str> s(n);
    Aho aho;
    for (int i = 0; i < K; i++) {
        s[i] = 'a' + i;
        aho.add_string(s[i], i);
    }
    for (int i = K; i < n; i++) {
        cin >> s[i];
        reverse(s[i].begin(), s[i].end());
        aho.add_string(s[i], i);
    }
    DSU dsu(n);
    while (k--) {
        int x, y;
        cin >> x >> y;
        x--, y--;
        dsu.unite(x, y);
    }
    aho.calculate();
    vector<vector<Hash>> hashes(n);
    {
        vi order(n);
        iota(order.begin(), order.end(), 0);
        sort(order.begin(), order.end(), [&] (int x, int y) {
            return s[x].size() < s[y].size();
        });
        for (int iter = 0; iter < n; iter++) {
            int i = order[iter];
            if (s[i].size() == 1) {
                hashes[i].resize(1);
                hashes[i][0] = Hash(dsu.get(i) + 1, 1);
                continue;
            }
            vi lp(s[i].size());
            vector<vector<BinUp>> up(s[i].size() + 1, vector<BinUp>(LG));
            up[0][0].to = 0;
            up[0][0].hash.hash = 0;
            up[0][0].hash.sz = 0;
            int cur = 0;
            for (int j = 0; j < s[i].size(); j++) {
                cur = aho.go(cur, s[i][j]);
                int x = aho.get_dp(cur);
                if (j + 1 == s[i].size()) {
                    x = aho.get_dp(aho.get_link(cur));
                }
                lp[j] = x;
                up[j + 1][0].to = j + 1 - s[x].size();
                up[j + 1][0].hash.hash = dsu.get(x) + 1;
                up[j + 1][0].hash.sz = 1;
            }
            for (int j = 1; j < LG; j++) {
                for (int q = 0; q <= s[i].size(); q++) {
                    up[q][j].to = up[up[q][j - 1].to][j - 1].to;
                    up[q][j].hash = unite(up[q][j - 1].hash, up[up[q][j - 1].to][j - 1].hash);
                }
            }
            hashes[i].resize(s[i].size());
            for (int q = 1; q < s[i].size(); q++) {
                Hash t1 = Hash();
                int a = q, b = s[i].size();
                for (int j = LG - 1; j >= 0 && a < b; j--) {
                    if (a <= up[b][j].to) {
                        t1 = unite(t1, up[b][j].hash);
                        b = up[b][j].to;
                    }
                }
                if (a < b) {
                    t1 = unite(t1, hashes[lp[b - 1]][s[lp[b - 1]].size() - (b - a)]);
                }
                hashes[i][q] = t1;
            }
            hashes[i][0] = Hash(dsu.get(i) + 1, 1);
        }
    }
    vi lp(t.size());
    vector<vector<BinUp>> up(t.size() + 1, vector<BinUp>(LG));
    {
        up[0][0].to = 0;
        up[0][0].hash.hash = 0;
        up[0][0].hash.sz = 0;
        int cur = 0;
        for (int i = 0; i < t.size(); i++) {
            cur = aho.go(cur, t[i]);
            int x = aho.get_dp(cur);
            lp[i] = x;
            up[i + 1][0].to = i + 1 - s[x].size();
            up[i + 1][0].hash.hash = dsu.get(x) + 1;
            up[i + 1][0].hash.sz = 1;
        }
        for (int j = 1; j < LG; j++) {
            for (int i = 0; i <= t.size(); i++) {
                up[i][j].to = up[up[i][j - 1].to][j - 1].to;
                up[i][j].hash = unite(up[i][j - 1].hash, up[up[i][j - 1].to][j - 1].hash);
            }
        }
    }
    int q;
    cin >> q;
    while (q--) {
        int a, b, c, d;
        cin >> a >> b >> c >> d;
        a = t.size() - a;
        b = t.size() - b;
        swap(a, b);
        b++;
        c = t.size() - c;
        d = t.size() - d;
        swap(c, d);
        d++;
        Hash t1 = Hash(), t2 = Hash();
        for (int iter = 0; iter < 2; iter++) {
            for (int i = LG - 1; i >= 0 && a < b; i--) {
                if (a <= up[b][i].to) {
                    t1 = unite(t1, up[b][i].hash);
                    b = up[b][i].to;
                }
            }
            if (a < b) {
                t1 = unite(t1, hashes[lp[b - 1]][s[lp[b - 1]].size() - (b - a)]);
            }
            swap(a, c);
            swap(b, d);
            swap(t1, t2);
        }
        if (t1.hash == t2.hash && t1.sz == t2.sz) {
            cout << "Yes\n";
        } else {
            cout << "No\n";
        }
    }
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);
    int test = 1;
    // cin >> test;
    precalc();
    while (test--) {
        solve();
    }
    return 0;
}
