//#pragma GCC optimize("Ofast,unroll-loops")
//#pragma GCC target("avx2,avx,sse2,sse")

#include <iostream>
#include <vector>
#include <map>
#include <cstdint>
#include <cassert>
#include <algorithm>
#include <queue>
#include <set>
#include <random>
#include <unordered_map>
#include <array>
#include <random>
#include <ctime>

using namespace std;
using ull = unsigned long long;
const int P = 300137;
const int MOD = 1e9 + 7;
struct Hash {
    ull h1;
    int h2;

    Hash() : h1(0), h2(0) {}
    Hash(int x) : h1(x), h2((x % MOD + MOD) % MOD) {}
    Hash(ull h1, int h2) : h1(h1), h2(h2) {}

    friend Hash operator+(const Hash& a, const Hash& b) {
        return {a.h1 + b.h1, (a.h2 + b.h2) % MOD};
    }

    friend Hash operator-(const Hash& a, const Hash& b) {
        return {a.h1 - b.h1, (a.h2 - b.h2 + MOD) % MOD};
    }

    friend Hash operator*(const Hash& a, const Hash& b) {
        return {a.h1 * b.h1, (int)((a.h2 * 1ll * b.h2) % MOD)};
    }

    Hash& operator+=(const Hash& other) {
        h1 += other.h1;
        h2 = (h2 + other.h2) % MOD;
        return *this;
    }

    Hash& operator-=(const Hash& other) {
        h1 -= other.h1;
        h2 = (h2 - other.h2 + MOD) % MOD;
        return *this;
    }

    Hash& operator*=(const Hash& other) {
        h1 *= other.h1;
        h2 = (h2 * 1ll * other.h2) % MOD;
        return *this;
    }

    friend bool operator==(const Hash& a, const Hash& b) {
        return a.h1 == b.h1 && a.h2 == b.h2;
    }
};

struct aho_korasik {
    struct node {
        node *to[26];
        node *go[26];
        node *suf_link = nullptr;
        node *parent = nullptr;
        char how;
        int len = 0;
        int idx = -1;
        node *closest_t = nullptr;

        node() {
            fill(begin(to), end(to), nullptr);
            fill(begin(go), end(go), nullptr);
        }
    };
    node* root = new node();

    void add(string &s, int idx) {
        node *v = root;
        for (char c : s) {
            if (!v->to[c - 'a']) {
                v->to[c - 'a'] = new node();
                v->to[c - 'a']->parent = v;
                v->to[c - 'a']->how = c;
            }
            v = v->to[c - 'a'];
        }
        v->len = s.size();
        v->idx = idx;
    }

    node *get_go(node *v, char c) {
        if (v->go[c - 'a'])
            return v->go[c - 'a'];
        if (v->to[c - 'a'])
            return v->go[c - 'a'] = v->to[c - 'a'];
        if (v == root)
            return v->go[c - 'a'] = root;
        node *tmp = get_suf_link(v);
        node *lol = get_go(tmp, c);
        return v->go[c - 'a'] = lol;
    }

    node *get_suf_link(node *v) {
        if (v->suf_link)
            return v->suf_link;
        if (v == root || v->parent == root)
            return v->suf_link = root;
        node *tmp = get_suf_link(v->parent);
        node *lol = get_go(tmp, v->how);
        return v->suf_link = lol;
    }

    void delete_tree(node *v) {
        for (auto u : v->to) {
            if (u)
                delete_tree(u);
        }
        delete v;
    }

    ~aho_korasik() {
        delete_tree(root);
    }

    node *closest_terminal(node *v) {
        if (v->closest_t) return v->closest_t;
        if (v->idx >= 0) return v->closest_t = v;
        return v->closest_t = closest_terminal(get_suf_link(v));
    }
};

const int N = 3e5 + 5;
Hash pw[(int)1e6 + 5];
vector<int> val;
string t;
vector<string> a;
vector<pair<int, int>> to;
vector<vector<Hash>> prec;

void fill_to() {
    aho_korasik reversed;
    for (int i = 0; i < a.size(); ++i) {
        auto s = a[i];
        reverse(s.begin(), s.end());
        reversed.add(s, i);
    }
    auto v = reversed.root;
    int sz = t.size();
    for (int i = sz - 1; i >= 0; --i) {
        v = reversed.get_go(v, t[i]);
        auto w = reversed.closest_terminal(v);
        to[i] = {i + w->len, w->idx};
    }
}

void solve() {
    cin >> t;
    int n, k; cin >> n >> k;
    n += 26;
    a.resize(n);
    prec.resize(n);
    val.assign(n, 0);
    to.resize(t.size());
    for (int i = 26; i < n; ++i) {
        cin >> a[i];
    }
    for (int i = 0; i < 26; ++i) {
        a[i] = string(1, 'a' + i);
    }
    vector<vector<int>> g(n);
    for (int i = 0; i < k; ++i) {
        int x, y; cin >> x >> y;
        --x, --y;
        g[x].push_back(y);
        g[y].push_back(x);
    }

    int nxt = 0;
    auto dfs = [&](int v, auto&& self) -> void {
        val[v] = nxt;
        for (int to : g[v]) {
            if (val[to] == 0) {
                self(to, self);
            }
        }
    };
    for (int i = 0; i < n; ++i) {
        if (!val[i]) {
            ++nxt;
            dfs(i, dfs);
        }
    }

    fill_to();
    aho_korasik helper;
    for (int i = 0; i < n; ++i) helper.add(a[i], i);
    for (int i = 0; i < n; ++i) {
        prec[i].resize(a[i].size());
        auto v = helper.get_go(helper.root, a[i][0]);
        vector<bool> was_here(a[i].size());
        vector<int> was_pos, was_idx(a[i].size(), -1);
        prec[i][0] = val[a[i][0] - 'a'];
        was_here[0] = true;
        was_pos.push_back(0);
        was_idx[0] = a[i][0] - 'a';
        for (int j = 1; j + 1 < a[i].size(); ++j) {
            v = helper.get_go(v, a[i][j]);
            auto w = v;
            while (true) {
                w = helper.closest_terminal(w);
                if (w->len == 1) {
                    prec[i][j] = prec[i][j - 1] + pw[was_pos.size()] * val[w->idx];
                    was_here[j] = true;
                    was_pos.push_back(j);
                    was_idx[j] = a[i][j] - 'a';
                    break;
                }
                if (was_here[j - w->len + 1]) {
                    prec[i][j] = prec[i][j - 1];
                    while (true) {
                        int p = was_pos.back();
                        prec[i][j] -= pw[was_pos.size() - 1] * val[was_idx[p]];
                        if (p == j - w->len + 1)
                            break;
                        was_here[p] = false;
                        was_pos.pop_back();
                        was_idx[p] = -1;
                    }
                    was_idx[was_pos.back()] = w->idx;
                    prec[i][j] += pw[was_pos.size() - 1] * val[w->idx];
                    break;
                }
                w = w->parent;
            }
        }
    }

    int q; cin >> q;
    vector<pair<int, int>> queries(q * 2);
    vector<Hash> ans(q * 2);
    for (auto& [l, r] : queries) {
        cin >> l >> r;
        --l, --r;
    }
    const int K = 20;
    vector<vector<int>> up(t.size(), vector<int>(K));
    vector<vector<Hash>> up_hash(t.size(), vector<Hash>(K));
    for (int i = 0; i < t.size(); ++i) {
        up[i][0] = to[i].first;
        up_hash[i][0] = val[to[i].second];
    }
    for (int j = 1; j < K; ++j) {
        for (int i = 0; i < t.size(); ++i) {
            if (up[i][j - 1] == t.size()) {
                up[i][j] = t.size();
                up_hash[i][j] = up_hash[i][j - 1];
            }
            else {
                up[i][j] = up[up[i][j - 1]][j - 1];
                up_hash[i][j] = up_hash[i][j - 1] + pw[1 << (j - 1)] * up_hash[up[i][j - 1]][j - 1];
            }
        }
    }

    for (int i = 0; i < q * 2; ++i) {
        auto [l, r] = queries[i];
        Hash res = 0;
        int len = 0;
        for (int j = K - 1; j >= 0; --j) {
            if (up[l][j] <= r + 1) {
                res += pw[len] * up_hash[l][j];
                len += (1 << j);
                l = up[l][j];
                if (l == r + 1)
                    break;
            }
        }
        if (l <= r) {
            assert(to[l].first >= r + 2);
            res += pw[len] * prec[to[l].second][r - l];
        }
        ans[i] = res;
    }

    for (int i = 0; i < q; ++i) {
        if (ans[i * 2] == ans[i * 2 + 1]) cout << "Yes\n";
        else cout << "No\n";
    }
}

int32_t main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    //freopen("javacert.in", "r", stdin);
    //freopen("javacert.out", "w", stdout);

    pw[0] = 1;
    for (int i = 1; i < (int)1e6 + 5; ++i) pw[i] = pw[i - 1] * P;
    solve();
}
